/*
 * main.c
 *
 * Copyright (C) 2004-2008 Atmark Techno, Inc.	All Rights Reserved.
 *
 */

#include <ctype.h>
#include "xuartlite_l.h"
#include "version.h"
#include "memmap.h"
#include "srec.h"
#include "medium.h"
#include "spi.h"
#define LED_GPIO(v)	(*(volatile unsigned long *)(LED_REGISTER_BASEADDR) = (v))
#define LED_ON		(0x01)
#define LED_OFF		(0x00)

#define MAX_BUFFER_SIZE (128)
//#define BOOTLOADER_OFFSET_SPI (0x00100000)
//#define BOOTLOADER_OFFSET_SPI (0x00180000)
#define BOOTLOADER_OFFSET_SPI   (0x00120000)
//#define BOOTLOADER_OFFSET_SPI   (0x0000000)


static void busy_wait(int time)
{
	volatile int i;

	for (i = 0; i < time; i++) {}
}

static int download(void)
{
	char buffer[MAX_BUFFER_SIZE];
	char *ptr1, *ptr2;
	int ret;

	while (1) {
		get_line(buffer, MAX_BUFFER_SIZE);

		/* Chew through any leading white space */
		ptr1 = buffer;
		while (*ptr1 && isspace(*ptr1))
			ptr1++;

		/* And truncate any trailing white space */
		ptr2 = ptr1;
		while (*ptr2 && !isspace(*ptr2))
			ptr2++;
		*ptr2 = '\0';

		put_char('.');
		/* check finish code */
		ret = srec_decode_line(ptr1);
		if (ret != SREC_DOWNLOAD_CONTINUE)
			return ret;
	}
}

//static unsigned int is_autoboot_mode(void)
//{
///	*(volatile unsigned int *)(SYSTEM_REGISTER_BASEADDR) =
//		SYSTEM_REGISTER_BOOT_MODE;
//	return *(volatile unsigned int *)(SYSTEM_REGISTER_BASEADDR) &
//		SYSTEM_REGISTER_BOOT_MODE;
//}

static int get_bootloader_offset(unsigned int *offset)
{
	int ret = 0;
	*offset = BOOTLOADER_OFFSET_SPI;
	return ret;
}

static void second_bootloader(unsigned int offset)
{
	myprint("Loading ");
	spi_copy_to_dram(offset, (void *)(BOOTLOADER_LOAD_ADDR), BOOTLOADER_SIZE);
	myprint("done. jump to...\r\n");
	jump_to_second_bootloader();	/* Never returns */
}

static void clear_rx_fifo(void)
{
	XUartLite_SetControlReg(UART_BASEADDR, XUL_CR_FIFO_RX_RESET);
	XUartLite_SetControlReg(UART_BASEADDR, 0);
}

int main(void)
{
	unsigned int bootloader_offset;
	char	key;

	//LED_GPIO(LED_OFF);

	busy_wait(10000);

	if (get_bootloader_offset(&bootloader_offset) < 0)
		goto halt;

//	if (is_autoboot_mode())
//		second_bootloader(bootloader_offset);

	myprint("\r\n\r\n"BBOOT_NAME " v" BBOOT_VERSION " (" \
		TARGET_CPU ") compiled at " __TIME__ ", " __DATE__ "\r\n");
	myprint("Press 'z' or 'Z' for BBoot Menu.\r\n");

	/* busy loop to wait getting a char 'z' or 'Z' for about 2sec */
	busy_wait(BBOOTMENU_WAITTIME);
	if (XUartLite_IsReceiveEmpty(UART_BASEADDR) ||
	    ((key = get_char()) != 0 && key != 'z' && key != 'Z'))
		second_bootloader(bootloader_offset);

	/* clear for long time pushing */
	clear_rx_fifo();

	myprint("\r\n\r\nPlease choose one of the following and hit enter.\r\n");
	myprint("a: activate second stage bootloader (default)\r\n");
	myprint("s: download a s-record file\r\n");

	while (1) {
		key = get_char();
		switch (key) {
		case 'a': /* activate second stage bootloader */
		case 'A':
		case '\r':
		case '\n':
			second_bootloader(bootloader_offset);
			break;
		case 's':
		case 'S':
			myprint("Start sending S-Record!!\r\n");
			if (download() < 0) {
				myprint("checksum mismatch.\r\n");
				goto halt;
			}
			break;
		default:
			myprint("Invalid selection.\r\n");
		case 'z':
		case 'Z':
			clear_rx_fifo();
			break;
		}
	}

halt:
	myprint("Halting...\r\n");
	return 0;
}
